@@ -38,8 +38,6 @@
 
 #if HAVE_AFINET6
 
-#include <netinet/ip6.h>
-
 /*
  *	This is in linux/include/net/ipv6.h.
  */
@@ -69,6 +67,11 @@ struct in6_ifreq {
 
 #endif  /* HAVE_AFINET6 */
 
+#ifndef IFF_PORTSEL
+#define IFF_PORTSEL	0x2000
+#define IFF_AUTOMEDIA	0x4000
+#endif
+
 /* This is from <linux/netdevice.h>. */
 
 struct user_net_device_stats
@@ -126,6 +129,18 @@ struct interface {
   struct user_net_device_stats stats;		/* statistics		 */
 };
 
+static const char *if_port_text[][4] = {
+  /* Keep in step with <linux/netdevice.h> */
+  { "unknown", NULL , NULL, NULL },
+  { "10base2", "bnc", "coax", NULL },
+  { "10baseT", "utp", "tpe", NULL },
+  { "AUI", "thick", "db15", NULL },
+  { "100baseT", NULL, NULL, NULL },
+  { "100baseTX", NULL, NULL, NULL },
+  { "100baseFX", NULL, NULL, NULL },
+  { NULL, NULL, NULL, NULL },
+};
+
 /* Not all kernels have these new ioctls. */
 #ifndef SIOGIFNAME
 #define SIOGIFNAME	0x8934		/* if_index -> name mapping	*/
@@ -219,8 +234,11 @@ ife_print(struct interface *ptr)
   if (hw->sprint != NULL && (strncmp(ptr->name, "atm", 3) || 
    (ptr->hwaddr[0] || ptr->hwaddr[1] || ptr->hwaddr[2] || ptr->hwaddr[3] || 
    ptr->hwaddr[4] || ptr->hwaddr[5]))) 
-    printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_hw, "HWaddr %s")
+    printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_hw, "HWaddr %s  ")
 	   , hw->print(ptr->hwaddr));
+  if (ptr->flags & IFF_PORTSEL) 
+    printf("Media:%s%s", if_port_text[ptr->map.port][0], 
+	   (ptr->flags & IFF_AUTOMEDIA)?"(auto)":"");
   printf("\n");
 #if HAVE_AFINET6  
   if (ap->af != AF_INET6) {
@@ -239,7 +257,7 @@ ife_print(struct interface *ptr)
 	   ap->sprint(&ptr->netmask, 1));
 #if HAVE_AFINET6
   }
-  if ((f = fopen("/proc/net/if_inet6", "r")) != NULL) {
+  if ((f = fopen(_PATH_PROCNET_IFINET6, "r")) != NULL) {
     while(fscanf(f, "%4s%4s%4s%4s%4s%4s%4s%4s %02x %02x %02x %02x %s\n",
 		 addr6p[0], addr6p[1], addr6p[2], addr6p[3],
 		 addr6p[4], addr6p[5], addr6p[6], addr6p[7],
@@ -427,13 +445,15 @@ static void if_getstats(char *ifname, struct interface *ife)
 
 /* Support for fetching an IPX address */
 
+#if HAVE_AFIPX
 static int ipx_getaddr(int sock, int ft, struct ifreq *ifr)
 {
 	((struct sockaddr_ipx *)&ifr->ifr_addr)->sipx_type=ft;
 	return ioctl(sock, SIOCGIFADDR, ifr);
 }
+#endif
 
-/* Fetch the inteface configuration from the kernel. */
+/* Fetch the interface configuration from the kernel. */
 static int
 if_fetch(char *ifname, struct interface *ife)
 {
@@ -645,27 +665,25 @@ usage(void)
 {
   fprintf(stderr, NLS_CATGETS(catfd, ifconfigSet, ifconfig_usage1,
 			      "Usage: ifconfig [-a] [-i] [-v] interface\n"));
+  fprintf(stderr, "                [[family] address]\n");
 #if HAVE_AFINET6
   fprintf(stderr, "                [add inet6address/prefixlen]\n");
   fprintf(stderr, "                [del inet6address/prefixlen]\n");
+  fprintf(stderr, "                [tunnel aa.bb.cc.dd]\n");
 #endif
 #if HAVE_AFINET
-  fprintf(stderr, NLS_CATGETS(catfd, ifconfigSet, ifconfig_usage2,
-			      "                [inet address]\n"));
   fprintf(stderr, "                [[-]broadcast [aa.bb.cc.dd]]\n");
   fprintf(stderr, "                [[-]pointopoint [aa.bb.cc.dd]]\n");
   fprintf(stderr, "                [netmask aa.bb.cc.dd]\n");
   fprintf(stderr, "                [dstaddr aa.bb.cc.dd]\n");
 #endif
-#if HAVE_AFAX25
-  fprintf(stderr, NLS_CATGETS(catfd, ifconfigSet, ifconfig_usage3,
-			      "                [hw] [ax25 address]\n"));
-#endif
+  fprintf(stderr, "                [hw class address]\n");
   fprintf(stderr, "                [metric NN] [mtu NN]\n");
   fprintf(stderr, "                [[-]trailers] [[-]arp]\n");
   fprintf(stderr, "                [[-]allmulti] [[-]promisc]\n");
   fprintf(stderr, "                [multicast]\n");
   fprintf(stderr, "                [mem_start NN] [io_addr NN] [irq NN]\n");
+  fprintf(stderr, "                [media type]\n");
   fprintf(stderr, "                [up] [down] ...\n");
   NLS_CATCLOSE(catfd)
   exit(1);
@@ -816,6 +834,44 @@ main(int argc, char **argv)
       continue;
     }
 
+    if (!strcmp(*spp, "media") || !strcmp(*spp, "port")) {
+      if (*++spp == NULL) usage();
+      if (!strcasecmp(*spp, "auto")) {
+	goterr |= set_flag(ifr.ifr_name, IFF_AUTOMEDIA);
+      } else {
+	int i, j, newport;
+	char *endp;
+	newport = strtol(*spp, &endp, 10);
+	if (*endp != 0) {
+	  newport = -1;
+	  for (i = 0; if_port_text[i][0] && newport == -1; i++) {
+	    for (j = 0; if_port_text[i][j]; j++) {
+	      if (!strcasecmp(*spp, if_port_text[i][j])) {
+		newport = i;
+		break;
+	      }
+	    }
+	  }
+	}
+	spp++;
+	if (newport == -1) {
+	  fprintf(stderr, "Unknown media type.\n");
+	  goterr = 1;
+	} else {
+	  if (ioctl(skfd, SIOCGIFMAP, &ifr) < 0) {
+	    goterr = 1;
+	    continue;
+	  }
+	  ifr.ifr_map.port = newport;
+	  if (ioctl(skfd, SIOCSIFMAP, &ifr) < 0) {
+	    fprintf(stderr, "SIOCSIFMAP: %s\n", strerror(errno));
+	    goterr = 1;
+	  }
+	}
+      }
+      continue;
+    }
+    
     if (!strcmp(*spp, "trailers")) {
       goterr |= clr_flag(ifr.ifr_name, IFF_NOTRAILERS);
       spp++;
@@ -1043,9 +1099,10 @@ main(int argc, char **argv)
     if (!strcmp(*spp, "hw")) {
       if (*++spp == NULL) usage();
       if ((hw = get_hwtype(*spp)) == NULL) usage();
-      strcpy(host, *++spp);
+      if (*++spp == NULL) usage();
+      strcpy(host, *spp);
       if (hw->input(host, &sa) < 0) {
-	ap->herror(host);
+	fprintf(stderr, "%s: invalid %s address.\n", host, hw->name);
 	goterr = 1;
 	spp++;
 	continue;
